import { Observable, forkJoin } from 'rxjs';
import { take } from 'rxjs/operators';

import { Injectable, Inject } from '@angular/core';
import {
  Resolve,
  ActivatedRouteSnapshot,
  RouterStateSnapshot
} from '@angular/router';

import { Store, select } from '@ngrx/store';

import { StatusCode } from '@ucap/api';
import { NativeService } from '@ucap/native';
import { SSOMode } from '@ucap/protocol-authentication';

import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { SessionStorageService } from '@ucap/ng-web-storage';
import { PublicApiService } from '@ucap/ng-api-public';
import { ExternalApiService } from '@ucap/ng-api-external';
import { ProtocolService } from '@ucap/ng-protocol';
import { InnerProtocolService } from '@ucap/ng-protocol-inner';
import { AuthenticationProtocolService } from '@ucap/ng-protocol-authentication';
import { QueryProtocolService } from '@ucap/ng-protocol-query';
import { OptionProtocolService } from '@ucap/ng-protocol-option';

import { CompanyActions } from '@ucap/ng-store-organization';
import { LoginActions } from '@ucap/ng-store-authentication';
import {
  BuddyActions,
  BuddySelector,
  GroupActions,
  GroupSelector
} from '@ucap/ng-store-group';
import { RoomActions, RoomSelector } from '@ucap/ng-store-chat';

import { LoginSession } from '@app/models/login-session';
import { AppKey } from '@app/types/app-key.type';

import { AppAuthenticationActions } from '@app/store/actions';

@Injectable()
export class AppAuthenticationResolver implements Resolve<void> {
  constructor(
    private publicApiService: PublicApiService,
    private externalApiService: ExternalApiService,
    private protocolService: ProtocolService,
    private innerProtocolService: InnerProtocolService,
    private authenticationProtocolService: AuthenticationProtocolService,
    private queryProtocolService: QueryProtocolService,
    private optionProtocolService: OptionProtocolService,
    private store: Store<any>,
    private sessionStorageService: SessionStorageService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService
  ) {}

  resolve(
    route: ActivatedRouteSnapshot,
    state: RouterStateSnapshot
  ): void | Observable<void> | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        const loginSession = this.sessionStorageService.get<LoginSession>(
          AppKey.LoginSession
        );
        const networkInfo = await this.nativeService.getNetworkInfo();
        const localIp =
          !!networkInfo && 0 < networkInfo.length && !!networkInfo[0].ip
            ? networkInfo[0].ip
            : '';
        const localMac =
          !!networkInfo && 0 < networkInfo.length && !!networkInfo[0].mac
            ? networkInfo[0].mac
            : '';

        this.store.dispatch(
          CompanyActions.companies({
            req: { companyGroupCode: loginSession.companyGroupCode }
          })
        );

        this.publicApiService
          .versionInfo2({
            deviceType: loginSession.deviceType,
            desktopType: loginSession.desktopType,
            companyGroupType: loginSession.companyGroupType,
            companyCode: loginSession.companyCode,
            loginId: loginSession.loginId
          })
          .pipe(take(1))
          .subscribe(
            versionInfo2Res => {
              if (StatusCode.Fail === versionInfo2Res.statusCode) {
                this.store.dispatch(
                  AppAuthenticationActions.versionInfo2Failure({
                    error: versionInfo2Res.errorMessage
                  })
                );

                reject(versionInfo2Res.errorMessage);
                return;
              }

              this.store.dispatch(
                AppAuthenticationActions.versionInfo2Success({
                  res: versionInfo2Res
                })
              );

              this.protocolService
                .connect(versionInfo2Res.serverIp)
                .pipe(take(1))
                .subscribe(
                  () => {
                    this.externalApiService
                      .urlInfo({
                        deviceType: loginSession.deviceType,
                        loginId: loginSession.loginId
                      })
                      .pipe(take(1))
                      .subscribe(
                        urlInfoRes => {
                          if (StatusCode.Fail === urlInfoRes.statusCode) {
                            this.store.dispatch(
                              AppAuthenticationActions.urlInfoFailure({
                                error: versionInfo2Res.errorMessage
                              })
                            );

                            reject(urlInfoRes.errorMessage);
                            return;
                          }

                          this.store.dispatch(
                            AppAuthenticationActions.urlInfoSuccess({
                              res: versionInfo2Res
                            })
                          );

                          this.innerProtocolService
                            .conn({})
                            .pipe(take(1))
                            .subscribe(
                              connRes => {
                                this.authenticationProtocolService
                                  .login({
                                    loginId: loginSession.loginId,
                                    loginPw: loginSession.loginPw,
                                    deviceType: loginSession.deviceType,
                                    deviceId: ' ',
                                    token: '',
                                    localeCode: loginSession.localeCode,
                                    pushId: ' ',
                                    companyCode: loginSession.companyCode,
                                    passwordEncodingType: 1,
                                    clientVersion: '',
                                    reconnect: false,
                                    ip: localIp,
                                    hostName: localMac,
                                    ssoMode: SSOMode.AUTH,
                                    userSpecificInformation: 'PRO_000482',
                                    andId: '',
                                    andPushRefreshYn: ''
                                  })
                                  .pipe(take(1))
                                  .subscribe(
                                    loginRes => {
                                      this.store.dispatch(
                                        LoginActions.loginSuccess({ loginRes })
                                      );

                                      forkJoin([
                                        this.queryProtocolService
                                          .auth({
                                            deviceType: loginSession.deviceType
                                          })
                                          .pipe(take(1)),
                                        this.optionProtocolService
                                          .regView({})
                                          .pipe(take(1))
                                      ])
                                        .pipe(take(1))
                                        .subscribe(
                                          ([authRes, regViewRes]) => {
                                            forkJoin([
                                              this.store.pipe(
                                                take(1),
                                                select(
                                                  BuddySelector.buddySyncDate
                                                )
                                              ),
                                              this.store.pipe(
                                                take(1),
                                                select(
                                                  GroupSelector.groupSyncDate
                                                )
                                              ),
                                              this.store.pipe(
                                                take(1),
                                                select(
                                                  RoomSelector.roomsSyncDate
                                                )
                                              )
                                            ])
                                              .pipe(take(1))
                                              .subscribe(
                                                ([
                                                  buddySyncDate,
                                                  groupSyncDate,
                                                  roomsSyncDate
                                                ]) => {
                                                  this.store.dispatch(
                                                    BuddyActions.buddy2({
                                                      req: {
                                                        syncDate: buddySyncDate
                                                      }
                                                    })
                                                  );
                                                  this.store.dispatch(
                                                    GroupActions.groups({
                                                      req: {
                                                        syncDate: groupSyncDate
                                                      }
                                                    })
                                                  );
                                                  this.store.dispatch(
                                                    RoomActions.rooms({
                                                      req: {
                                                        localeCode:
                                                          loginSession.localeCode,
                                                        syncDate: roomsSyncDate
                                                      }
                                                    })
                                                  );
                                                }
                                              );

                                            resolve();
                                          },
                                          error => {
                                            reject(error);
                                          }
                                        );
                                    },
                                    error => {
                                      this.store.dispatch(
                                        LoginActions.loginFailure({ error })
                                      );
                                      reject(error);
                                    }
                                  );
                              },
                              error => {
                                reject(error);
                              }
                            );
                        },
                        error => {
                          this.store.dispatch(
                            AppAuthenticationActions.urlInfoFailure({
                              error
                            })
                          );
                          reject(error);
                        }
                      );
                  },
                  error => {
                    reject(error);
                  }
                );
            },
            error => {
              this.store.dispatch(
                AppAuthenticationActions.versionInfo2Failure({
                  error
                })
              );
              reject(error);
            }
          );
      } catch (error) {
        reject(error);
      }
    });
  }
}
